﻿/*
# #############################################################################
#  ATTENTION
# #############################################################################
#
#  This software is for guidance only. It is provided "as is".
#  It only aims at providing coding information to the customer in order for
#  them to save time. As a result no warranties, whether express, implied or
#  statutory, including, but not limited to implied warranties of
#  merchantability and fitness for a particular purpose apply to this software.
#  A·P·E Angewandte Physik und Elektronik GmbH shall not be held liable for any
#  direct, indirect or consequential damages with respect to any claims arising
#  from the content of such sourcecode and/or the use made by customers of the
#  coding information contained herein in connection with their products.
#
#  File Version: 1.0
#
#  Changelog:
#    1.0 - Initial Version
#
# #############################################################################
*/

using System;
using System.IO;
using System.Text;
using System.Collections.Generic;
using System.Globalization;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;


namespace APE_Interface_Example
{
    public class APE_Interface_Example
    {
        public static int Main(String[] args)
        {
            //By default C# uses the "," as decimal separator in some countries (aspecially inside the EU)
            //To fix this, the following three commands set the "." as decimal separator for the application.
            System.Globalization.CultureInfo customCulture = (System.Globalization.CultureInfo)System.Threading.Thread.CurrentThread.CurrentCulture.Clone();
            customCulture.NumberFormat.NumberDecimalSeparator = ".";
            System.Threading.Thread.CurrentThread.CurrentCulture = customCulture;

            //Create new APE_Device Object.
            //Class Constructor will automatically try to connect to the given remote target
            APE_Device device = new APE_Device("localhost", 51123, "APEDevice");
            if (!device.is_connected())
            {
                Console.WriteLine("Connection failed.");
                Console.ReadKey();
                return 1;
            }

            Console.WriteLine("Connected.\n");

            //Get Device IDN
            Console.WriteLine("SCPI Device IDN: ");
            Console.WriteLine(device.query("*IDN?\r\n"));

            /*******************************************************************
             * The following code demonstrates how to obtain unpack and display
             * the ACF function from an APE pulseCheck or miniUSB Autocorrelator
             * 
             * The same functionality can also be used to unpack the various
             * other trace-data streams returned by other APE software 
             * (e.g. waveScan Spectra)
             ********************************************************************/

                        //Read data from the device and convert it into a byte array for unpacking
            //"True" parameter enables the automatic SCPI Block unpacking functionality
            //The returned data will be a raw binary data string that must be unpacked
            string data = device.query(":ACF:DATA?", true);

            //Unpack the string into single bytes by using the System.Text.Encoding.GetBytes() function.
            //Important! Do NOT use the ASCII encoder, since it only uses 7-Bit characters! The unpacked data won't be valid!
            //You MUST use a 8-bit wide codepage for decoding.
            Encoding enc = System.Text.Encoding.GetEncoding(1252);
            byte[] unpacked = enc.GetBytes(data);

            //Calculate the number of data points per axis: Array length / 8 Bytes per DBL / 2 Axis
            int len = unpacked.Length / 8 / 2;

            //Create new empty arrays to store the converted double values for both axis
            double[] xScopeData = new double[len];
            double[] yScopeData = new double[len];
           
            //Run through the byte array, take 8 byte-blocks and convert them into double values using BitConverter
            //Bit Converter reads the required amount of bytes from the specified array from the given offset position
            //and converts them into the selected data type (in this case it reads 8 bytes for each double value)
            for (int i = 0; i < len; i++)
            {
                //Read first 8 bytes from offset for X-Value
                xScopeData[i] = BitConverter.ToDouble(unpacked, i * 16);

                //Move by 8 Bytes to unpack the following Y-Value
                yScopeData[i] = BitConverter.ToDouble(unpacked, i * 16 + 8);
            }

            //Create Form with graph control (from Form1.cs) and get a reference to the chart object
            Form1 plot_window = new Form1();
            Chart chart1 = plot_window.xyplot;

            //Clear the plot window and then set the graph title
            chart1.Series.Clear();
            chart1.Titles.Add("ACF Data");

            //Add a data series to the chart and set its type to a line-plot
            Series plot_data = chart1.Series.Add("ACF");
            chart1.Series["ACF"].ChartType = SeriesChartType.Line;

            //Add Points to the chart and find min/max values for the x-Axis
            double max_x = 0;
            double min_x = 0;
            for (int i = 0; i < xScopeData.Length; i++)
            {
                plot_data.Points.AddXY(xScopeData[i], yScopeData[i]);

                if (xScopeData[i] < min_x)
                    min_x = xScopeData[i];

                if (xScopeData[i] > max_x)
                    max_x = xScopeData[i];
            }

            //Set X-Axis Scale to +/- 0.5ps above/below the first and last value in the trace
            chart1.ChartAreas[0].AxisX.Maximum = Math.Round(max_x) + 0.5;
            chart1.ChartAreas[0].AxisX.Minimum = Math.Round(min_x) - 0.5;

            //Add labels to the Axis
            chart1.ChartAreas[0].AxisX.Title = "Delay (ps)";
            chart1.ChartAreas[0].AxisY.Title = "Intensity (arb.u.)";

            //Update the chart window
            chart1.Update();

            //Open the windows and pause execution until the window is closed
            plot_window.ShowDialog();

 
            //Close TCP Socket connection to the device, since we won't send or receive any more data
            device.disconnect();

            return 0;
        }
    }
}